@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2005-2023 Intel Corporation
+    Copyright (c) 2005-2025 Intel Corporation
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -65,7 +65,7 @@ struct alignas(max_nfs_size) arena_slot_private_state {
     /** Modified by the owner thread (during these operations). **/
     unsigned hint_for_fifo_stream;
 
-#if __TBB_PREVIEW_CRITICAL_TASKS
+#if __TBB_CRITICAL_TASKS
     //! Similar to 'hint_for_fifo_stream' but for critical tasks.
     unsigned hint_for_critical_stream;
 #endif
@@ -188,12 +188,12 @@ public:
 #if __TBB_RESUMABLE_TASKS
         hint_for_resume_stream = h;
 #endif
-#if __TBB_PREVIEW_CRITICAL_TASKS
+#if __TBB_CRITICAL_TASKS
         hint_for_critical_stream = h;
 #endif
     }
 
-#if __TBB_PREVIEW_CRITICAL_TASKS
+#if __TBB_CRITICAL_TASKS
     unsigned& critical_hint() {
         return hint_for_critical_stream;
     }
